'FMA Script Framework Core Class
'ManagedMenu
'Easy interface to implement a menu in your plugin

'TODO:
'-property let/get for the list!?
'-am.Clear vs. am.ClearMenu
'-Tweak max menu items. Title may be shorter than 15ch, prev/next item isn't always needed

'Menu constraints:
'-AT command string must not be >250 bytes/chars (AT*EASM="<title>",1,<selecteditem>,<numitems>,"<item1>","<item2>",...,"<itemn>")
'-Menu title must not be >15 bytes/chars
'-The selected item must not be > number of items

'Const MENU_NEXT_PAGE              = "Next Page >>"
'Const MENU_PREV_PAGE              = "<< Prev Page"
Const MENU_MAX_TITLE_LENGTH       = 15
Const MENU_AT_COMMAND_MAX_LENGTH  = 250
'Const MENU_AT_COMMAND_BASE_LENGTH = 63 'AT*EASM="123456789012345",1,99,99,"<< Prev Page","Next Page >>"
Const MENU_AT_COMMAND_BASE_LENGTH = 33 'AT*EASM="123456789012345",1,99,99
Const MENU_AT_COMMAND_ITEM_LENGTH = 3  ',""

Public MenuStack
Set MenuStack = New Stack

Class ManagedMenu
	
	Private m_Title
	Private List
	Private Page
	Private Pages()
	Private numPages
	Private isRoot
	
	Sub Class_Initialize ()
		isRoot = 0
		'Init an empty menu
		m_Title = ""
		Set List = New LinkedList
	End Sub
	
	Public Property Let Title (ByVal t)
		m_Title = Util.EscapeInvalidATChars(t)
	End Property
	Public Property Get Title
		Title = m_Title
	End Property
	
	Sub SetList(lst)
		Set List = lst
		
		'Split list into pages. We have to split the list so that the AT command isn't >250 bytes/chars
		Dim ATLength, bi
		numPages = 0
		ReDim Pages(0)
		'Init ATLengt with maxlength to force the creation of the first page
		ATLength = MENU_AT_COMMAND_MAX_LENGTH
		
	  Dim menuIt, menuLast, entry
	  'escape entries
		menuIt   = List.Begin
	  menuLast = List.Last
	  Do Until menuIt.Object Is menuLast.Object
			menuIt.Item = Array(Util.EscapeInvalidATChars(menuIt.Item.Item(0)), menuIt.Item.Item(1))
			menuIt.Iterate()
	  Loop
	  'Split list
		menuIt   = List.Begin
	  menuLast = List.Last
	  Do Until menuIt.Object Is menuLast.Object
			If ATLength + MENU_AT_COMMAND_ITEM_LENGTH + Len(menuIt.Item.Item(0)) > MENU_AT_COMMAND_MAX_LENGTH Then
				'Max AT command size exceeded. Create new list
				numPages = numPages + 1
				ReDim Preserve Pages(numPages - 1)
				Set Pages(numPages - 1) = New LinkedList
				'Now insert in the next page
				bi = Pages(numPages - 1).BackInserter
				'Init ATLength
				ATLength = MENU_AT_COMMAND_BASE_LENGTH
			End If
			'Add next item
			Set bi.Item = menuIt.Item
			ATLength = ATLength + MENU_AT_COMMAND_ITEM_LENGTH + Len(menuIt.Item.Item(0))
			menuIt.Iterate()
	  Loop
		
		'Init Menu
		Page = 1
	End Sub
	
	Sub ShowMenu()
		'Push menu if menu stack is empty or the top-menu is unequal to the menu which shall be pushed
		If MenuStack.IsEmpty Then
			MenuStack.Push Me
			isRoot = 1
			'Only register the joystick navigation once!
			KeyManager.RegisterKey KEY_JOYLEFT,  "MenuStack.Top.PrevPage", STATE_PRESS, Me
			KeyManager.RegisterKey KEY_JOYRIGHt, "MenuStack.Top.NextPage", STATE_PRESS, Me
		Else
			If Not (Me Is MenuStack.Top) Then 'Don't push the same menu twice
				MenuStack.Push Me
			End If
		End If
		MenuStack.Top.Show
	End Sub
	
	Sub Show()
		'Don't show empty menus
		If List.Count > 0 Then
			am.Clear
			Dim pagesTitle, cutTitle
			
			If numPages > 1 Then
				pagesTitle = " (" & Page & "/" & numPages & ")"
				'am.AddItem MENU_PREV_PAGE, "MenuStack.Top.PrevPage"
			End If
			cutTitle = Left(Title, MENU_MAX_TITLE_LENGTH - Len(pagesTitle))
			If Right(cutTitle, 1) = "\" And Right(cutTitle, 2) <> "\\" Then cutTitle = Left(cutTitle, Len(cutTitle) - 1) 'cut trailing \ which may result when cutting the title
			am.Title = cutTitle & pagesTitle 'A menu title must not be > 15 chars!
			
			If numPages > 0 Then
				' LinkedList version. Note the menuIt/menuLast assignment without Set!
				Dim menuIt, menuLast
				menuIt   = Pages(Page - 1).Begin
				menuLast = Pages(Page - 1).Last
				Do Until menuIt.Object Is menuLast.Object
					am.AddItem menuIt.Item.Item(0), menuIt.Item.Item(1)
					menuIt.Iterate()
				Loop
				'If numPages > 1 Then am.AddItem MENU_NEXT_PAGE, "MenuStack.Top.NextPage"
			End If
		End If
		
		'Capture back-button action
		am.Back = "MenuStack.Top.Quit"
		
		'Use "wait" as general next state. So the phone won't fall back to the
		'connection -> accessories -> FMA menu between menu updates.
		'Don't use this for the root menu! Otherwise we cannot quit the menu anymore
		If isRoot <> 1 Then
			am.NextState = 2 'Wait
		Else
			am.NextState = 1 'Previous
		End If
	  am.Update
	End Sub
	
	Sub NextPage()
		If numPages > 1 Then
			Page = Page + 1
			If Page > numPages Then Page = 1
			Show
		End If
	End Sub
	
	Sub PrevPage()
		If numPages > 1 Then
			Page = Page - 1
			If Page < 1 Then Page = numPages
			Show
		End If
	End Sub
	
	Sub ShowPage( num )
		If num > numPages Then num = numPages
		If num < 1 Then num = 1
		Page = num
		ShowMenu
	End Sub
	
	Function GetPageAndIndexByTitleSubstr( title )
		Dim p, i
		title = Util.EscapeInvalidATChars(title)
		GetPageAndIndexByTitleSubstr = 1 'Failsafe
		For p = 1 To numPages
			Dim menuIt, menuLast
			menuIt   = Pages(p - 1).Begin
			menuLast = Pages(p - 1).Last
			i = 0
			Do Until menuIt.Object Is menuLast.Object
				If InStr(menuIt.Item.Item(0), title) > 0 Then 'Item found
					GetPageAndIndexByTitleSubstr = Array(p, i)
					Exit For
				End If
				menuIt.Iterate()
				i = i + 1
			Loop
		Next
	End Function
	
	Function GetPageByTitleSubstr( title )
		GetPageByTitleSubstr = Me.GetPageAndIndexByTitleSubstr(title)(0)
	End Function
	
	Function GetPageAndIndexByItemIndex( idx )
		Dim p, i, j
		GetPageAndIndexByItemIndex = 1 'Failsafe
		j = 0
		For p = 1 To numPages
			Dim menuIt, menuLast
			menuIt   = Pages(p - 1).Begin
			menuLast = Pages(p - 1).Last
			i = 0
			Do Until menuIt.Object Is menuLast.Object
				If j = idx Then 'Item found
					GetPageAndIndexByItemIndex = Array(p, i)
					Exit For
				End If
				menuIt.Iterate()
				i = i + 1
				j = j + 1
			Loop
		Next
	End Function
	
	Function GetPageByItemIndex( idx )
		GetPageByItemIndex = GetPageAndIndexByItemIndex(idx)(0)
	End Function
	
	Sub Quit()
		EventManager.OnEvent "MenuClose", Array(m_Title)
		If isRoot = 1 Then KeyManager.DeregisterAll Me 'Only deregister joystick navigation when the mainmenu quits
		MenuStack.Pop
		'Cannot show not existing menu
		If Not MenuStack.IsEmpty Then MenuStack.Top.Show
	End Sub

End Class


