unit uCalling;

{
*******************************************************************************
* Descriptions: Calling/Called Popup Implementation
* $Source: /cvsroot/fma/fma/uCalling.pas,v $
* $Locker:  $
*
* Todo:
*   - Update to support note taking for the active caller
*
* Change Log:
* $Log: uCalling.pas,v $
* Revision 1.8  2004/06/08 19:19:25  lordlarry
* Memory Leak fixed
*
* Revision 1.7  2003/12/11 14:08:02  z_stoichev
* Fixed Command return error on answer.
* Timer and sound start adjusted to match phone ones.
* Handle some possible exceptions.
*
* Revision 1.6  2003/11/28 09:38:07  z_stoichev
* Merged with branch-release-1-1 (Fma 0.10.28c)
*
* Revision 1.5.2.8  2003/11/21 13:34:11  z_stoichev
* Fixed headset button mark call as missed issue.
*
* Revision 1.5.2.7  2003/11/14 15:41:02  z_stoichev
* Updates for patch 27d.
*
* Revision 1.5.2.6  2003/11/13 16:35:39  z_stoichev
* Fixed personalization support.
*
* Revision 1.5.2.5  2003/11/12 16:48:52  z_stoichev
* Do not show error on missing sound file.
*
* Revision 1.5.2.4  2003/11/12 15:19:03  z_stoichev
* Temporary disconnect phone (during call) support.
* Image auto-scale to fit.
*
* Revision 1.5.2.3  2003/11/11 18:11:06  z_stoichev
* Add background image.
* Show contact picture and play sound
* if personalized in phonebook.
*
* Revision 1.5.2.2  2003/10/31 14:51:15  z_stoichev
* Added headset button (disconnect on answer).
*
* Revision 1.5.2.1  2003/10/27 07:22:54  z_stoichev
* Build 0.1.0 RC1 Initial Checkin.
*
* Revision 1.5  2003/10/15 15:49:55  z_stoichev
* MIssed Calls Unicode support.
* GUI changes.
*
* Revision 1.4  2003/02/17 06:51:16  crino77
* Added support for missed calls
*
* Revision 1.3  2003/01/30 04:15:57  warren00
* Updated with header comments
*
*
*******************************************************************************
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Placemnt, TntStdCtrls, GR32_Image, ExtCtrls, MPlayer,
  jpeg;

type
  TfrmCalling = class(TForm)
    HandupButton: TButton;
    AnswerButton: TButton;
    FormPlacement1: TFormPlacement;
    lbAlpha: TTntLabel;
    lbNumber: TTntLabel;
    HeadsetButton: TButton;
    Panel1: TPanel;
    Image32: TImage32;
    MediaPlayer1: TMediaPlayer;
    Image1: TImage;
    lblTime: TLabel;
    TimeTimer: TTimer;
    procedure CallButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MediaPlayer1Notify(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure TimeTimerTimer(Sender: TObject);
  private
    { Private declarations }
    FStartTime: TDateTime;
    FPersonalized,FStopped: boolean;
    FIsIncoming: Boolean;
    procedure Set_IsIncoming(const Value: Boolean);
  public
    { Public declarations }
    FCheck: Boolean;
    FListCalls: TStringList;
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    procedure DoPersonalize;
    procedure StopPersonalize(HidePic: boolean = True);
    function HasPersonalizedSound: boolean;
  published
    property IsIncoming: Boolean read FIsIncoming write Set_IsIncoming;
  end;

var
  frmCalling: TfrmCalling;

implementation

uses Unit1, uSyncPhonebook;

{$R *.dfm}

constructor TfrmCalling.Create(Owner: TComponent);
begin
   inherited Create(Owner);
   FListCalls := TStringList.Create;  { TODO -oLordLarry : Who fills this list? Doesn't seem to do anything }
end;

procedure TfrmCalling.CallButtonClick(Sender: TObject);
begin
  if Sender = HandupButton then begin
    StopPersonalize;
    try Form1.VoiceHangUp;
    except end;
  end
  else
  if Sender = AnswerButton then begin
    StopPersonalize(False);
    try Form1.VoiceAnswer;
    except end; // Error may occur is call is alerady active...
    FCheck := True;
  end
  else
  if Sender = HeadsetButton then begin
    StopPersonalize(False);
    try Form1.VoiceAnswer;
    except end;
    FCheck := True;
    Form1.DoDisconnectTemporary;
  end;
  (Sender As TButton).Enabled := False;
end;

procedure TfrmCalling.FormShow(Sender: TObject);
begin
  FStopped := False;
  FPersonalized := False;
  FCheck := False;
  AnswerButton.Enabled := True;
  HandupButton.Enabled := True;
  HeadsetButton.Enabled := True;
  SetWindowPos(Handle, HWND_TOPMOST,
    Top, Left, Width, Height,
    SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
  lblTime.Caption := '00:00';
end;

procedure TfrmCalling.DoPersonalize;
var
  contact: PContactData;
  s: string;
begin
  if not FPersonalized and form1.frmSyncPhonebook.FindContact(lbAlpha.Caption,contact) then begin
    FPersonalized := True;
    if IsIncoming then
      Form1.ShowBaloonInfo(GetContactFullName(contact)+' is calling...',60);
    try
      Image32.Visible := False;
      s := GetContactPictureFile(contact);
      if s <> '' then begin
        Image32.Bitmap.LoadFromFile(s);
        Image32.Visible := True;
      end;
    except
    end;
    WaitASec(500); // delay 1/2 sec to match phone sound :)
    try
      s := GetContactSoundFile(contact);
      if IsIncoming and (s <> '') then begin
        MediaPlayer1.FileName := s;
        MediaPlayer1.Open;
        MediaPlayer1.Play;
        MediaPlayer1.Notify := True;
      end;
    except
      MediaPlayer1.FileName := '';
    end;
    FStopped := False;
  end;
end;

procedure TfrmCalling.MediaPlayer1Notify(Sender: TObject);
begin
  { loop sound }
  if FPersonalized and not FStopped and (MediaPlayer1.Mode = mpStopped) then
    try
      MediaPlayer1.Play;
      MediaPlayer1.Notify := True;
    except
    end;
end;

procedure TfrmCalling.StopPersonalize(HidePic: boolean);
begin
  if FStopped then exit;
  FStopped := True;
  if HidePic then
    Image32.Visible := False
  else begin
    TimeTimer.Enabled := True;
    FStartTime := Now;
  end;
  if MediaPlayer1.FileName <> '' then try
    MediaPlayer1.Notify := False;
    if IsIncoming then
      try
        MediaPlayer1.Stop;
        MediaPlayer1.Close;
      except
      end;
  except
  end;
  Form1.CoolTrayIcon1.HideBalloonHint;
end;

procedure TfrmCalling.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  StopPersonalize;
  { Close sound file! }
  if MediaPlayer1.FileName <> '' then try
    MediaPlayer1.Close;
  except
  end;
end;

procedure TfrmCalling.FormCreate(Sender: TObject);
begin
  Image1.Picture.Assign(Form1.BackgrImage.Picture);
end;

procedure TfrmCalling.FormHide(Sender: TObject);
begin
  TimeTimer.Enabled := False;
end;

procedure TfrmCalling.TimeTimerTimer(Sender: TObject);
begin
  lblTime.Caption := FormatDateTime('nn:ss',Now - FStartTime);
end;

procedure TfrmCalling.Set_IsIncoming(const Value: Boolean);
begin
  FIsIncoming := Value;
  if Visible then
    if Value then Form1.ShowBaloonInfo('Incoming call...',60);
end;

function TfrmCalling.HasPersonalizedSound: boolean;
begin
  Result := FPersonalized and (MediaPlayer1.FileName <> '');
end;

destructor TfrmCalling.Destroy;
begin
  FListCalls.Free;
  
  inherited;
end;

end.

