unit uFolderProps;

{
*******************************************************************************
* Descriptions: Implementation for Explorer properties click
* $Source: /cvsroot/fma/fma/uFolderProps.pas,v $
* $Locker:  $
*
* Todo:
*      - use notebook in general page
*
* Change Log:
* $Log: uFolderProps.pas,v $
* Revision 1.8  2004/05/19 18:34:15  z_stoichev
* Build 0.1.0.35c
*
* Revision 1.7  2004/03/26 18:37:39  z_stoichev
* Build 0.1.0.35 RC5
*
* Revision 1.6  2004/03/08 12:54:22  z_stoichev
* Phone number - msg/call contact
* Autoupdate phone details every 1 sec
*
* Revision 1.5  2003/12/16 17:38:05  z_stoichev
* Refresh gives error when folder does not exists.
*
* Revision 1.4  2003/12/11 14:55:24  z_stoichev
* Fixed Explorer properties visual bug issues.
* Phone properties now shown more phone information.
*
* Revision 1.3  2003/12/11 12:41:02  z_stoichev
* Refresh obex file implemented.
*
* Revision 1.2  2003/11/28 09:38:07  z_stoichev
* Merged with branch-release-1-1 (Fma 0.10.28c)
*
* Revision 1.1.2.4  2003/11/13 16:39:54  z_stoichev
* Add more items type support.
*
*
*
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, TntComCtrls, TntStdCtrls,
  GR32_Image, MPlayer, OKTThemeEngine;

type
  TfrmFolderProps = class(TForm)
    ApplyButton: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Bevel1: TBevel;
    Image1: TImage;
    lblName: TLabel;
    TntEdit1: TTntEdit;
    TabSheet2: TTabSheet;
    lblDB: TLabel;
    Memo1: TMemo;
    TabSheet3: TTabSheet;
    PicPanel1: TPanel;
    SelImage: TImage32;
    MediaPlayer1: TMediaPlayer;
    Label1: TLabel;
    lblType: TLabel;
    lblNoCache: TLabel;
    btnDownload: TButton;
    lblNote: TLabel;
    Label3: TLabel;
    InfoPanel: TPanel;
    Label13: TLabel;
    Label12: TLabel;
    lbModel: TLabel;
    lbManufacturer: TLabel;
    Label14: TLabel;
    Label16: TLabel;
    lbSWRevision: TLabel;
    lbSerialNumber: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lbvbat: TLabel;
    lbicharge: TLabel;
    lbcyclescharge: TLabel;
    Label7: TLabel;
    lbdcio: TLabel;
    lbiphone: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lblTimeLeft: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    lbtempphone: TLabel;
    lbtempbatt: TLabel;
    Timer1: TTimer;
    pnContact: TPanel;
    Button1: TButton;
    Button2: TButton;
    lblContact: TLabel;
    ThemeViewer1: TOKTThemeViewer;
    ThemeContainer1: TOKTThemeContainer;
    btnSS: TButton;
    cbPreview: TComboBox;
    SaveDialog1: TSaveDialog;
    lblLocalFile: TLabel;
    procedure ApplyButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MediaPlayer1Click(Sender: TObject; Button: TMPBtnType;
      var DoDefault: Boolean);
    procedure btnDownloadClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure cbPreviewChange(Sender: TObject);
    procedure btnSSClick(Sender: TObject);
  private
    Filetype: integer;
    Fullpath: string;
    FRootNode: TtntTreeNode;
    procedure Set_RootNode(const Value: TtntTreeNode);
    { Private declarations }
    procedure UpdatePhoneDetails;
  public
    { Public declarations }
    property RootNode: TtntTreeNode read FRootNode write Set_RootNode default nil;
  end;

var
  frmFolderProps: TfrmFolderProps;

implementation

uses Unit1;

{$R *.dfm}

procedure TfrmFolderProps.ApplyButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmFolderProps.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmFolderProps.Set_RootNode(const Value: TtntTreeNode);
var
  id: Integer;
begin
  FRootNode := Value;
  if Value = nil then begin
    Memo1.Visible := False;
    lblDB.Visible := False;
    exit;
  end;

  { Allow phone rename if connected and root is selected }
  InfoPanel.Visible := (Value = Form1.Explorer.Items[0]);
  if InfoPanel.Visible then begin
    TntEdit1.ReadOnly := False;
    TntEdit1.Color := clWindow;
    TntEdit1.Text := Value.Text;
    { Get phone info }
    UpdatePhoneDetails;
  end
  else 
    TntEdit1.Visible := False;

  if TabSheet2.TabVisible then begin
    lblName.Caption := Value.Text;
    Memo1.Lines.Clear;
    Memo1.Visible := True;
    lblDB.Visible := True;
    id := Value.StateIndex;

    if Form1.PhoneIdentity <> '' then
      Fullpath := ExePath+'data\'+Form1.PhoneIdentity+'\dat\'
    else
      Fullpath := ExePath;

    // Contacts
    if (id and $F00000) = $100000 then begin
      Memo1.Lines.Add(Fullpath+'Contacts.ME.dat');
      Memo1.Lines.Add(Fullpath+'Contacts.SM.dat');
      Memo1.Lines.Add(Fullpath+'Contacts.SYNC.dat');
      Memo1.Lines.Add(Fullpath+'Contacts.SYNCMAX.dat');
    end;
    // Messages
    if (id and $F00000) = $200000 then begin
      Memo1.Lines.Add(Fullpath+'SMSInbox.dat');
      Memo1.Lines.Add(Fullpath+'SMSSent.dat');
      Memo1.Lines.Add(Fullpath+'SMSArchive.dat');
    end;
    // Files
    if (id and $F00000) = $500000 then Memo1.Lines.Add('Dynamic');
    // Profiles
    if (id and $F00000) = $700000 then Memo1.Lines.Add('Dynamic');
    // Groups
    if (id and $F00000) = $800000 then Memo1.Lines.Add('Dynamic');

    // Phones
    if Value.ImageIndex in [9..13] then begin
      lblContact.Caption := Value.Parent.Text;
      pnContact.Visible := True;
    end
    else
      pnContact.Visible := False;

    if Memo1.Lines.Count = 0 then begin
      TabSheet2.TabVisible := False;
    end;
  end;  

  if TabSheet3.TabVisible then begin
    Filetype := -1;
    id := Form1.ExplorerFindExtImage(ExtractFileExt(Value.Text));
    try
      lblLocalFile.Caption := '';
      case id of
        36: // sound
        begin
          Fullpath := ExePath+'data\'+Form1.PhoneIdentity+'\snd\'+Value.Text;
          if FileExists(Fullpath) then begin
            lblNoCache.Caption := 'cached localy';
            MediaPlayer1.FileName := Fullpath;
            MediaPlayer1.Open;
            MediaPlayer1.Enabled := True;
            MediaPlayer1.Visible := True;
            lblLocalFile.Caption := Fullpath;
          end
          else
            lblNoCache.Caption := 'stored in phone';
            btnDownload.Visible := True;
            btnDownload.Enabled := True;
            lblType.Caption := 'sound';
            Filetype := 1;
          end;
        37: // image
        begin
          Fullpath := ExePath+'data\'+Form1.PhoneIdentity+'\pic\'+Value.Text;
          if FileExists(Fullpath) then begin
            lblNoCache.Caption := 'cached localy';
            SelImage.Bitmap.LoadFromFile(Fullpath);
            SelImage.Visible := True;
            lblLocalFile.Caption := Fullpath;
          end
          else
            lblNoCache.Caption := 'stored in phone';
          btnDownload.Visible := True;
          btnDownload.Enabled := True;
          PicPanel1.Visible := True;
          lblType.Caption := 'picture';
          Filetype := 0;
        end;
        27: // theme
        begin
          Fullpath := ExePath+'data\'+Form1.PhoneIdentity+'\thm\'+Value.Text;
          if FileExists(Fullpath) then begin
            lblNoCache.Caption := 'cached localy';
            ThemeContainer1.OpenTheme(Fullpath);
            lblLocalFile.Caption := Fullpath;
            ThemeViewer1.Preview := pvStandby;
            ThemeViewer1.Visible := True;
            cbPreview.ItemIndex := Ord(pvStandby)-1;
            cbPreview.Visible := True;
            btnSS.Visible := True;
          end
          else
            lblNoCache.Caption := 'stored in phone';
          btnDownload.Visible := True;
          btnDownload.Enabled := True;
          lblType.Caption := 'theme';
          Filetype := 2;
        end;
      end;
    except
      lblNoCache.Caption := 'file damaged, or unknown format';
      btnDownload.Visible := True;
      btnDownload.Enabled := True;
    end;
    if not PicPanel1.Visible and not MediaPlayer1.Visible and (Filetype = -1) then begin
      TabSheet3.TabVisible := False;
    end;
  end;  
end;

procedure TfrmFolderProps.FormCreate(Sender: TObject);
begin
{$IFNDEF VER150}
  Form1.ThemeManager1.CollectForms(Self);
{$ENDIF}
end;

procedure TfrmFolderProps.FormShow(Sender: TObject);
begin
  PageControl1.ActivePageIndex := 0;
end;

procedure TfrmFolderProps.MediaPlayer1Click(Sender: TObject;
  Button: TMPBtnType; var DoDefault: Boolean);
begin
  if Button = btStop then MediaPlayer1.Rewind;
end;

procedure TfrmFolderProps.btnDownloadClick(Sender: TObject);
begin
  btnDownload.Enabled := False;
  ForceDirectories(ExtractFileDir(Fullpath));
  Form1.ObexGetFile(Fullpath,Form1.FindObexFolderName(Filetype)+'/'+ExtractFileName(Fullpath));
  RootNode := RootNode;
end;

procedure TfrmFolderProps.UpdatePhoneDetails;
begin
  lbManufacturer.Caption   := Form1.frmInfoView.lbManufacturer.Caption;
  lbModel.Caption          := Form1.frmInfoView.lbModel.Caption;
  lbSWRevision.Caption     := Form1.frmInfoView.lbSWRevision.Caption;
  lbSerialNumber.Caption   := Form1.frmInfoView.lbSerialNumber.Caption;
  lbvbat.Caption           := Form1.frmInfoView.lbvbat.Caption;
  lbicharge.Caption        := Form1.frmInfoView.lbicharge.Caption;
  lbcyclescharge.Caption   := Form1.frmInfoView.lbcyclescharge.Caption;
  lbdcio.Caption           := Form1.frmInfoView.lbdcio.Caption;
  lbiphone.Caption         := Form1.frmInfoView.lbiphone.Caption;
  lblTimeLeft.Caption      := Form1.frmInfoView.lblTimeLeft.Caption;
  lbtempbatt.Caption       := Form1.frmInfoView.lbtempbatt.Caption;
  lbtempphone.Caption      := Form1.frmInfoView.lbtempphone.Caption;
  Timer1.Enabled := True;
end;

procedure TfrmFolderProps.Timer1Timer(Sender: TObject);
begin
  UpdatePhoneDetails;
end;

procedure TfrmFolderProps.cbPreviewChange(Sender: TObject);
begin
  ThemeViewer1.Preview := TOKTPreview(cbPreview.ItemIndex+1);
end;

procedure TfrmFolderProps.btnSSClick(Sender: TObject);
begin
  SaveDialog1.FileName := ChangeFileExt(lblName.Caption,'') + ' - ' + cbPreview.Text + '.bmp';
  if SaveDialog1.Execute then
    ThemeViewer1.ScreenShot.SaveToFile(SaveDialog1.FileName);
end;

end.
