unit uSIMEdit;

{
*******************************************************************************
* Descriptions: SIM Card Phonebook Edit Implementation
* $Source: /cvsroot/fma/fma/uSIMEdit.pas,v $
* $Locker:  $
*
* Todo:
*
* Change Log:
* $Log: uSIMEdit.pas,v $
* Revision 1.13  2004/05/21 10:09:05  z_stoichev
* Changed logging handle routines.
*
* Revision 1.12  2004/05/19 18:34:16  z_stoichev
* Build 0.1.0.35c
*
* Revision 1.11  2004/03/26 18:37:40  z_stoichev
* Build 0.1.0.35 RC5
*
* Revision 1.10  2004/03/07 21:55:45  z_stoichev
* Apply length restrictions when copy ME to/from SM.
* Sort all in SIM changes misplaced contacts only.
*
* Revision 1.9  2004/01/30 16:42:16  z_stoichev
* Fixed Send contacts from SIM to Phone AV error.
* Added Progress dialog on Uploading contacts.
* Added Fma windows update on SMS Delete/Upload.
*
* Revision 1.8  2004/01/28 17:35:36  z_stoichev
* Popup menu rearranged.
* Allow editing Phone memory.
*
* Revision 1.7  2003/12/12 16:54:24  z_stoichev
* Added view customization support.
*
* Revision 1.6  2003/11/28 09:38:07  z_stoichev
* Merged with branch-release-1-1 (Fma 0.10.28c)
*
* Revision 1.5.2.7  2003/11/26 12:27:18  z_stoichev
* Sort after new contact.
* Select All works now.
*
* Revision 1.5.2.6  2003/11/13 16:37:10  z_stoichev
* Changed images.
*
* Revision 1.5.2.5  2003/11/12 16:21:07  z_stoichev
* Allow contact properties from Explorer popup menu.
*
* Revision 1.5.2.4  2003/11/11 13:11:32  z_stoichev
* Fixed surname is not cleared on edit.
*
* Revision 1.5.2.3  2003/11/10 14:03:10  z_stoichev
* RC3
*
* Revision 1.5.2.2  2003/11/07 13:56:04  z_stoichev
* SIM view/editor recoded.
*
* Revision 1.5.2.1  2003/10/27 07:22:54  z_stoichev
* Build 0.1.0 RC1 Initial Checkin.
*
* Revision 1.5  2003/10/23 11:52:18  z_stoichev
* Fixed bug for max name and tel length.
* Popup menu recreated.
* Font changed.
*
* Revision 1.4  2003/10/21 11:59:31  z_stoichev
* Update SIM sanity checks added.
* Retrieve max name and tel length.
* Bugfixes.
*
* Revision 1.3  2003/01/30 04:15:57  warren00
* Updated with header comments
*
*
*******************************************************************************
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, ExtCtrls, StdCtrls, Math, Menus, VirtualTrees, ImgList, uSyncPhonebook,
  Placemnt;

type
  TSIMData = Record
    imageindex,position: integer;
    cname,pnumb: WideString;
    ptype: string[2];
  end;
  PSIMData = ^TSIMData;

  TfrmSIMEdit = class(TFrame)
    Panel1: TPanel;
    btnUpdateSIM: TButton;
    cbForce: TCheckBox;
    btnReset: TButton;
    PopupMenu1: TPopupMenu;
    Clear1: TMenuItem;
    Resetchangeflag1: TMenuItem;
    ListNumbers: TVirtualStringTree;
    NoItemsPanel: TPanel;
    N1: TMenuItem;
    N2: TMenuItem;
    Properties1: TMenuItem;
    NewPerson1: TMenuItem;
    N3: TMenuItem;
    UpdateChanged1: TMenuItem;
    UpdateAllRecords1: TMenuItem;
    FormStorage1: TFormStorage;
    N5: TMenuItem;
    UpdateContactsPosition1: TMenuItem;
    UpdateContactsfromPhonebook1: TMenuItem;
    SendAllContactstoPhonebook1: TMenuItem;
    N4: TMenuItem;
    ImportContacts1: TMenuItem;
    ExportContacts1: TMenuItem;
    N6: TMenuItem;
    procedure Button1Click(Sender: TObject);
    procedure StringGridGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure Resetchangeflag1Click(Sender: TObject);
    procedure ListNumbersAfterPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas);
    procedure ListNumbersCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure ListNumbersGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure ListNumbersGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure ListNumbersHeaderClick(Sender: TVTHeader;
      Column: TColumnIndex; Button: TMouseButton; Shift: TShiftState; X,
      Y: Integer);
    procedure btnUpdateSIMClick(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure btnDELClick(Sender: TObject);
    procedure btnEDITClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure NewPerson1Click(Sender: TObject);
    procedure UpdateChanged1Click(Sender: TObject);
    procedure UpdateAllRecords1Click(Sender: TObject);
    procedure UpdateContactsPosition1Click(Sender: TObject);
    procedure UpdateContactsfromPhonebook1Click(Sender: TObject);
    procedure SendAllContactstoPhonebook1Click(Sender: TObject);
    procedure ListNumbersIncrementalSearch(Sender: TBaseVirtualTree;
      Node: PVirtualNode; const SearchText: WideString;
      var Result: Integer);
  private
    { Private declarations }
    FMaxNumbers: cardinal;
    FMaxNameLen,FMaxTelLen: integer;
    FOldText: String;
    function GetSIMCapacity: Integer;
    function FindFreePos: integer;
    procedure UpdateSIM(MaxItems: cardinal = 0);
    procedure AddLog(Msg: string);
  public
    { Public declarations }
    FContact: TContactData;
    SelContact: PSIMData;
    FRendered: Boolean;
    constructor Create(AOwner: TComponent); override;
    procedure RenderData(ForceSMMode: boolean = False);
    procedure UpdatePhonebook;
    procedure OnConnected;
    function IsMEMode: boolean;
    function IsRendered: boolean;
    function FindContact(FullName: WideString; var AContact: PSIMData): boolean;
    function DoEdit(AsNew: boolean = False): boolean;
    procedure CheckForChanges;
  end;

implementation

uses Unit1, gsm_sms, uGlobal, uEditContact, uConnProgress;

{$R *.dfm}

{ TFrame2 }

constructor TfrmSIMEdit.Create(AOwner: TComponent);
begin
  inherited;
  { Set defaults }
  FMaxNameLen := 80; FMaxTelLen := 14; FMaxNumbers := 200;
  FRendered := False;
end;

procedure TfrmSIMEdit.RenderData(ForceSMMode: boolean);
var
  sl: TStrings;
  i: Integer;
  Item: PSIMData;
  Node: PVirtualNode;
  s, Name, Kind: WideString;
  Number: String;
  pos,state: Integer;
begin
  { ForceSMMode is used when loading Profile database }
  if not ForceSMMode and IsMEMode then sl := form1.FNodeContactsME.Data
    else sl := form1.FNodeContactsSM.Data;
  try
    ListNumbers.BeginUpdate;
    ListNumbers.Clear;
    ListNumbers.NodeDataSize := sizeof(TSIMData);
    try
      try
        for i := 0 to sl.Count - 1 do begin
          s := sl.Strings[i];
          {
          Name := GetToken(s, 0);
          Number := GetToken(s, 1);
          Pos := StrToInt(GetToken(s, 2));

          try
            state := StrToInt(GetToken(s, 3));
          except
            state := 3;
          end;
          }
          Name := GetFirstToken(s);
          Number := GetFirstToken(s);
          Pos := StrToInt(GetFirstToken(s));
          state := StrToInt(GetFirstToken(s));

          if state = 0 then Abort;

          Node := ListNumbers.AddChild(nil);
          item := ListNumbers.GetNodeData(Node);

          Form1.ExtractName(Name,Kind);

          with item^ do begin
            cname := Name;
            pnumb := Number;
            ptype := Kind;
            imageindex := state;
            position := Pos;
          end;
        end;

        FRendered := True;
      except
        ShowMessage('Old data format, please refresh the folder.');
      end;
    finally
      ListNumbers.EndUpdate;
      ListNumbers.Sort(nil, ListNumbers.Header.SortColumn, ListNumbers.Header.SortDirection);
    end;
  except
  end;  
end;

procedure TfrmSIMEdit.Button1Click(Sender: TObject);
begin
  if not Form1.FConnected then Form1.ActionConnectionConnectExecute(Self);

  Form1.ActionContactsDownloadExecute(Self);
  RenderData;
end;

procedure TfrmSIMEdit.UpdateSIM(MaxItems: cardinal);
var
  PerformCleanup: boolean;
  maxPos,LastPos,numType: Integer;
  name, number, buf: WideString;
  Item: PSIMData;
  Node: PVirtualNode;
  frmConnect: TfrmConnect;
begin
  PerformCleanup := MaxItems = ListNumbers.ChildCount[nil];
  Form1.RequestConnection;
  AddLog('Starting update...');
  frmConnect := GetProgressDialog;
  try
    if Form1.CanShowProgress then
      frmConnect.ShowProgress(Form1.FProgressLongOnly);
    if MaxItems > 0 then
      frmConnect.Initialize(MaxItems,'Updating phonebook')
    else
      frmConnect.SetDescr('Updating phonebook');
    Form1.Status('Uploading contacts...');
    maxPos := 0;
    try
      if IsMEMode then Form1.TxAndWait('AT+CPBS="ME"')
        else Form1.TxAndWait('AT+CPBS="SM"');
      try
        Node := ListNumbers.GetFirst;
        while Node <> nil do
        try
          item := ListNumbers.GetNodeData(Node);
          if (item.imageindex <> 3) or cbForce.Checked then begin
            numType := 129;
            Name := item.cname;
            Number := item.pnumb;
            if Form1.FUseUTF8 then Name := UTF8Encode(Name);

            if maxPos <= item.position then maxPos := item.position;

            buf := 'AT+CPBW=' + IntToStr(item.position);
            if Item.imageindex = 2 then begin
              // Delete
              AddLog('delete '+item.cname+' / '+item.pnumb);
              Form1.TxAndWait(buf);
            end
            else begin
              // Update
              if Number[1] = '+' then begin
                Number := copy(Number, 2, length(Number));
                numType := 145;
              end;
              buf := buf + ',"' + Number + '",' + IntToStr(numType) + ',"' + Name + '"';
              AddLog('update position '+IntToStr(item.position)+' as '+Name+' / '+Number);
              Form1.TxAndWait(buf);
              Item.imageindex := 3;
            end;
          end;
        finally
          frmConnect.IncProgress(1);
          Node := ListNumbers.GetNext(Node);
          Update;
        end;
        { Ususaly happens when you rearrange contacts positions, i.e. modify all of them }
        if PerformCleanup then begin
          Form1.Status('Performing cleanup...');
          frmConnect.SetDescr('Cleanup phonebook');
          frmConnect.SEProgress1.UnknownMax := True;
          if IsMEMode then LastPos := Form1.frmSyncPhonebook.FMaxRecME
            else LastPos := FMaxNumbers;
          for maxPos := maxPos + 1 to LastPos do begin
            buf := 'AT+CPBW=' + IntToStr(maxPos);
            Form1.TxAndWait(buf);
          end;
        end;
      finally
        UpdatePhonebook;
      end;
    except
      Form1.Status('Error updating memory');
      ShowMessage('Error updating memory');
      AddLog('Update finished with Error.');
    end;
    AddLog('Update finished successfuly.');
  finally
    FreeProgressDialog;
    Form1.Status('');
  end;  
end;

procedure TfrmSIMEdit.btnUpdateSIMClick(Sender: TObject);
var
  Item: PSIMData;
  Node: PVirtualNode;
  count: cardinal;
  s: string;
  err: boolean;
begin
  count := 0;

  Node := ListNumbers.GetFirst;
  while Node <> nil do
  try
    item := ListNumbers.GetNodeData(Node);
    if (item.imageindex <> 2) and // fix - do not check deleted entries 
      ((item.imageindex <> 3) or cbForce.Checked) then begin // then process modified (or all) entries
      if (Trim(item.cname) = '') and (Trim(item.pnumb) <> '') then
        raise EConvertError.Create('Please enter a contact name at position '+IntToStr(item.position));

      if (Trim(item.cname) <> '') and (Trim(item.pnumb) = '') then
        raise EConvertError.Create('Please enter a phone number at position '+IntToStr(item.position));

      if IsMEMode then err := cardinal(Length(item.cname)) > Form1.frmSyncPhonebook.FMaxNameLen
        else err := Length(item.cname) > FMaxNameLen;
      if err then raise EConvertError.Create('The name at position '+IntToStr(item.position)+' is too long');

      if IsMEMode then err := cardinal(Length(item.pnumb)) > (Form1.frmSyncPhonebook.FMaxTelLen+byte(Pos('+',item.pnumb) <> 0))
        else err := Length(item.pnumb) > (FMaxTelLen+byte(Pos('+',item.pnumb) <> 0));
      if err then raise EConvertError.Create('The number at position '+IntToStr(item.position)+' is too long');

      count := count + 1;
    end;
  finally
    Node := ListNumbers.GetNext(Node);
  end;

  if count <> 0 then begin
    if count = ListNumbers.ChildCount[nil] then s := 'All'
      else s := IntToStr(count) + ' modified';
    case MessageDlg('Changes have been made into current folder.'#13#13+
      'Confirm sending ' + s + ' contacts to phone?', mtConfirmation, [mbYes, mbNO, mbCANCEL], 0) of
      mrYes: UpdateSIM(count);
      mrCancel: Abort;
    end;
  end;
end;

procedure TfrmSIMEdit.btnResetClick(Sender: TObject);
var
  Item: PSIMData;
  Node: PVirtualNode;
begin
  ListNumbers.BeginUpdate;
  Node := ListNumbers.GetFirst;
  while Node <> nil do
  try
    item := ListNumbers.GetNodeData(Node);
    item.imageindex := 3; // mark as not modified
  finally
    Node := ListNumbers.GetNext(Node);
  end;
  ListNumbers.EndUpdate;
  UpdatePhonebook;
end;

procedure TfrmSIMEdit.StringGridGetEditText(Sender: TObject; ACol,
  ARow: Integer; var Value: String);
begin
  FOldText := Value;
end;

procedure TfrmSIMEdit.btnDELClick(Sender: TObject);
var
  Item: PSIMData;
  Node,Tmp: PVirtualNode;
  s: string;
begin
  if ListNumbers.SelectedCount = 0 then exit;
  s := 'Deleting ' + IntToStr(ListNumbers.SelectedCount) + ' item(s)';
  if MessageDlg(s+'. Do you wish to continue?',mtConfirmation,[mbYes,mbNo],0) <> ID_YES then
    exit;
  Form1.Status(s+'...');
  ListNumbers.BeginUpdate;
  try
    Node := ListNumbers.GetFirst;
    while Node <> nil do begin
      if ListNumbers.Selected[Node] then begin
        item := ListNumbers.GetNodeData(Node);
        if item.imageindex <> 0 then begin
          item.imageindex := 2;
        end
        else begin
          Tmp := Node;
          if Node <> ListNumbers.GetFirst then begin
            Node := ListNumbers.GetPrevious(Node);
            ListNumbers.DeleteNode(Tmp);
          end
          else begin
            ListNumbers.DeleteNode(Tmp);
            Node := ListNumbers.GetFirst;
            continue;
          end;
        end;
      end;
      Node := ListNumbers.GetNext(Node);
    end;
  finally
    ListNumbers.EndUpdate;
    UpdatePhonebook;
  end;  
  Form1.Status('');
end;

procedure TfrmSIMEdit.Resetchangeflag1Click(Sender: TObject);
var
  Item: PSIMData;
  Node: PVirtualNode;
begin
  ListNumbers.BeginUpdate;
  Node := ListNumbers.GetFirst;
  while Node <> nil do
  try
    if ListNumbers.Selected[Node] then begin
      item := ListNumbers.GetNodeData(Node);
      item.imageindex := 3;
    end;
  finally
    Node := ListNumbers.GetNext(Node);
  end;
  ListNumbers.EndUpdate;
  UpdatePhonebook;
end;

function TfrmSIMEdit.GetSIMCapacity: Integer;
var
  i: Integer;
  buffer, stop: String;
  slTmp: TStrings;
begin
  Form1.TxAndWait('AT+CPBS="SM"');
  Form1.TxAndWait('AT+CPBR=?');
  // defaults
  buffer := '';
  stop := '200'; FMaxNameLen := 80; FMaxTelLen := 14;
  // +CPBR: (1-200),80,14
  for i := 0 to Form1.FRxBuffer.Count-1 do
    if Pos('+CPBR',Form1.FRxBuffer.Strings[i]) = 1 then begin
      buffer := Form1.FRxBuffer.Strings[i];
      break;
    end;
  for i := 1 to length(buffer) do begin
    if IsDelimiter('()-,', buffer, i) then buffer[i] := ' ';
  end;
  // +CPBR:  1 200  80 14
  if buffer <> '' then begin
    slTmp := TStringList.Create;
    try
      slTmp.DelimitedText := buffer;
      stop := slTmp.Strings[2];
      Form1.Debug('SIM: max entries = '+stop);
      FMaxNameLen := StrToInt(slTmp.Strings[3]);
      Form1.Debug('SIM: max name length = '+slTmp.Strings[3]);
      FMaxTelLen := StrToInt(slTmp.Strings[4]);
      Form1.Debug('SIM: max tel length = '+slTmp.Strings[4]);
    finally
      slTmp.Free;
    end;
  end;  
  Result := StrToInt(stop);
end;

procedure TfrmSIMEdit.OnConnected;
begin
  FMaxNumbers := GetSIMCapacity;
end;

procedure TfrmSIMEdit.ListNumbersAfterPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas);
begin
  NoItemsPanel.Visible := ListNumbers.ChildCount[nil] = 0;
end;

procedure TfrmSIMEdit.ListNumbersCompareNodes(Sender: TBaseVirtualTree;
  Node1, Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  SIM1, SIM2: PSIMData;
begin
  SIM1 := Sender.GetNodeData(Node1);
  SIM2 := Sender.GetNodeData(Node2);

  if Column = 0 then begin
    if SIM1.position > SIM2.position then
      Result := 1
    else
      if SIM1.position < SIM2.position then
        Result := -1
      else
        Result := 0;  
  end
  else if Column = 1 then Result := WideCompareStr(SIM1.cname, SIM2.cname)
  else if Column = 2 then Result := WideCompareStr(SIM1.pnumb, SIM2.pnumb);
end;

procedure TfrmSIMEdit.ListNumbersGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
var
  SIM: PSIMData;
begin
  if Column = 0 then begin
    if (Kind = ikNormal) or (Kind = ikSelected) then begin
      SIM := Sender.GetNodeData(Node);
      ImageIndex := SIM.imageindex;
    end
    else ImageIndex := -1;
  end;
end;

procedure TfrmSIMEdit.ListNumbersGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  SIM: PSIMData;
begin
  SIM := Sender.GetNodeData(Node);

  if Column = 0 then CellText := IntToStr(SIM.position)
  else if Column = 1 then CellText := SIM.cname
  else if Column = 2 then CellText := SIM.pnumb;
end;

procedure TfrmSIMEdit.ListNumbersHeaderClick(Sender: TVTHeader;
  Column: TColumnIndex; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  if Button = mbLeft then begin
    if Column = Sender.SortColumn then begin
      if Sender.SortDirection = sdDescending then
        Sender.SortDirection := sdAscending
      else
        Sender.SortDirection := sdDescending;
    end
    else
      Sender.SortColumn := Column;
    ListNumbers.Sort(nil, ListNumbers.Header.SortColumn, ListNumbers.Header.SortDirection);
  end;
end;

procedure TfrmSIMEdit.btnEDITClick(Sender: TObject);
var
  Node :PVirtualNode;
begin
  Node := ListNumbers.FocusedNode;
  if Node <> nil then begin
    SelContact := ListNumbers.GetNodeData(Node);
    DoEdit;
  end;
end;

procedure TfrmSIMEdit.PopupMenu1Popup(Sender: TObject);
begin
  Properties1.Enabled := ListNumbers.SelectedCount = 1;
  SendAllContactstoPhonebook1.Enabled := not Form1.FStartupOptions.NoIRMC;
  SendAllContactstoPhonebook1.Visible := not IsMEMode;
  UpdateContactsfromPhonebook1.Enabled := SendAllContactstoPhonebook1.Enabled;
  UpdateContactsfromPhonebook1.Visible := SendAllContactstoPhonebook1.Visible;
  N3.Visible := SendAllContactstoPhonebook1.Visible;
end;

function TfrmSIMEdit.DoEdit(AsNew: boolean): boolean;
var
  Node: PVirtualNode;
  i: integer;
begin
  Result := False;
  with TfrmEditContact.Create(nil) do
  try
    // set restrictions
    MaxFullNameLen :=  FMaxNameLen;
    txtName.MaxLength := FMaxNameLen;
    txtCell.MaxLength := FMaxTellen;
    // update contact state
    if AsNew then FillChar(FContact,SizeOf(FContact),0)
    else begin
      { Set contact name and surname }
      i := Pos(' ',Selcontact^.cname);
      if i <> 0 then begin
        FContact.name := Copy(Selcontact^.cname,1,i-1);
        FContact.surname := Copy(Selcontact^.cname,i+1,Length(Selcontact^.cname)-i);
      end
      else begin
        FContact.name := Selcontact^.cname;
        FContact.surname := '';
      end;
      { Set number according to phone type }
      FContact.cell := '';
      FContact.work := '';
      FContact.home := '';
      FContact.fax := '';
      FContact.other := '';
      if Selcontact^.pnumb <> '' then
        if not IsMEMode or (Selcontact^.ptype = '') then
          FContact.cell := Selcontact^.pnumb
        else
          case Selcontact^.ptype[1] of
            'M': FContact.cell := Selcontact^.pnumb;
            'W': FContact.work := Selcontact^.pnumb;
            'H': FContact.home := Selcontact^.pnumb;
            'F': FContact.fax := Selcontact^.pnumb;
            'O': FContact.other := Selcontact^.pnumb;
          end;
    end;
    contact := FContact;
    UseSIMMode := True;  
    // edit contact as a SIM entry  
    if ShowModal = mrOk then begin
      if Modified then with ListNumbers do begin
        // apply total updates
        BeginUpdate;
        try
          if AsNew then begin // create new node
            Node := AddChild(nil);
            Selcontact := ListNumbers.GetNodeData(Node);
          end
          else
            Node := nil;  
          try
            { copy all data }
            FContact := contact;
            Selcontact^.cname := FContact.name;
            if FContact.surname <> '' then
              Selcontact^.cname := Selcontact^.cname + ' ' + FContact.surname;

            { get new number }
            if AsNew then begin
              if FContact.cell <> '' then begin
                Selcontact^.pnumb := FContact.cell;
                Selcontact^.ptype := 'M';
              end;
              if FContact.work <> '' then begin
                Selcontact^.pnumb := FContact.work;
                Selcontact^.ptype := 'W';
              end;
              if FContact.home <> '' then begin
                Selcontact^.pnumb := FContact.home;
                Selcontact^.ptype := 'H';
              end;
              if FContact.fax <> '' then begin
                Selcontact^.pnumb := FContact.fax;
                Selcontact^.ptype := 'F';
              end;
              if FContact.other <> '' then begin
                Selcontact^.pnumb := FContact.other;
                Selcontact^.ptype := 'O';
              end;
            end
            else
              if not IsMEMode or (Selcontact^.ptype = '') then
                Selcontact^.pnumb := FContact.cell
              else
                case Selcontact^.ptype[1] of
                  'M': Selcontact^.pnumb := FContact.cell;
                  'W': Selcontact^.pnumb := FContact.work;
                  'H': Selcontact^.pnumb := FContact.home;
                  'F': Selcontact^.pnumb := FContact.fax;
                  'O': Selcontact^.pnumb := FContact.other;
                end;

            if AsNew then begin
              Selcontact^.imageindex := 0;
              Selcontact^.position := FindFreePos;
              if Selcontact^.position = -1 then
                DeleteNode(Node)
              else
                Result := True;
            end
            else
              if Selcontact^.imageindex <> 0 then begin // mark as modified
                Selcontact^.imageindex := 1;
                Result := True;
              end;
          except;
            if AsNew then DeleteNode(Node);
            raise;
          end;
          UpdatePhonebook;
        finally
          EndUpdate;
        end;
      end; 
    end;
  finally
    Free;
  end;
  ListNumbers.Sort(nil, ListNumbers.Header.SortColumn, ListNumbers.Header.SortDirection);
end;

procedure TfrmSIMEdit.NewPerson1Click(Sender: TObject);
var
  OK: boolean;
begin
  if IsMEMode then
    OK := ListNumbers.RootNodeCount < Form1.frmSyncPhonebook.FMaxRecME
  else
    OK := ListNumbers.RootNodeCount < FMaxNumbers;
  if OK then
    DoEdit(True)
  else
    ShowMessage('No more space in memory! New contact can not be created.');
end;

procedure TfrmSIMEdit.UpdateChanged1Click(Sender: TObject);
begin
  cbForce.Checked := False;
  btnUpdateSIM.Click;
end;

procedure TfrmSIMEdit.UpdateAllRecords1Click(Sender: TObject);
begin
  cbForce.Checked := True;
  btnUpdateSIM.Click;
end;

procedure TfrmSIMEdit.AddLog(Msg: string);
begin
  Form1.SyncLog(Msg);
end;

function TfrmSIMEdit.FindFreePos: integer;
var
  Node :PVirtualNode;
  Item: PSIMData;
  Pos: cardinal;
  found: boolean;
begin
  Pos := 1;
  while Pos <= FMaxNumbers do begin
    found := false;
    Node := ListNumbers.GetFirst;
    while Node <> nil do
    try
      item := ListNumbers.GetNodeData(Node);
      if cardinal(item.position) = Pos then begin
        found := true;
        break;
      end;
    finally
      Node := ListNumbers.GetNext(Node);
    end;
    if not found then break;
    Pos := Pos + 1;
  end;
  if Pos <= FMaxNumbers then Result := Pos
    else Result := -1;
end;

function TfrmSIMEdit.FindContact(FullName: WideString;
  var AContact: PSIMData): boolean;
var
  Node :PVirtualNode;
begin
  Result := False;
  Node := ListNumbers.GetFirst;
  while Node <> nil do
  try
    AContact := ListNumbers.GetNodeData(Node);
    if WideCompareText(FullName,AContact^.cname) = 0 then begin
      Result := True;
      break;
    end;
  finally
    Node := ListNumbers.GetNext(Node);
  end;
end;

procedure TfrmSIMEdit.UpdateContactsPosition1Click(Sender: TObject);
var
  Pos: Integer;
  Item: PSIMData;
  Node: PVirtualNode;
begin
  if MessageDlg('Sorting contacts will replace all contacts position. Are you sure?',
    mtConfirmation, [mbYes, mbNO], 0) <> mrYes then exit;
  Pos := 1;
  try
    Node := ListNumbers.GetFirst;
    while Node <> nil do
    try
      item := ListNumbers.GetNodeData(Node);
      if item.position <> Pos then begin
        item.position := Pos;
        item.imageindex := 1; // mark as modified
      end;
      inc(Pos);
    finally
      Node := ListNumbers.GetNext(Node);
    end;
  finally
    ListNumbers.Sort(nil, ListNumbers.Header.SortColumn, ListNumbers.Header.SortDirection);
    UpdatePhonebook;
  end;  
end;

procedure TfrmSIMEdit.UpdateContactsfromPhonebook1Click(Sender: TObject);
var
  sl,dl: TStrings;
  i,j: integer;
  s,item,t: string;
begin
  sl := TStrings(Form1.FNodeContactsME.Data); // source
  dl := TStrings(Form1.FNodeContactsSM.Data); // destination

  if ListNumbers.ChildCount[nil] <> 0 then begin
    if sl.Count = 0 then begin
      if MessageDlg('Phonebook is empty. All current SIM entries will be deleted. Continue anyway?',
        mtConfirmation, [mbYes, mbNO], 0) <> mrYes then exit;
    end
    else
      if MessageDlg('Copy all contacts from Phonebook. Current SIM entries will be deleted. Are you sure?',
        mtConfirmation, [mbYes, mbNO], 0) <> mrYes then exit;
  end;

  { Clear SIM database }
  dl.Clear;
  Form1.FNodeContactsSM.DeleteChildren;
  try
    { Build new database }
    for i := 0 to sl.Count-1 do begin
      // '"Alycia/M",+359887555555555,0'
      { get full name }
      item := GetToken(sl.Strings[i],0);
      j := Length(item);
      if (j > 2) and (item[j-1] = '/') then Delete(item,j-1,2); // remove phone type, we can not keep it in SIM
      item := Copy(item,1,FMaxNameLen);
      { get number }
      t := GetToken(sl.Strings[i],1);
      s := Copy(t,1,FMaxTelLen+byte(Pos('+',t) <> 0));
      { Construct a database item }
      item := '"' + item + '",' + s + ',' + IntToStr(i+1) + ',1'; // and mark (1) as modified 
      { Add to SIM database }
      dl.Add(item);
    end;
  finally
    Form1.RenderContactList(Form1.FNodeContactsSM);
    { Update view }
    RenderData;
    Form1.Status('Copy from Phonebook: ' + IntToStr(ListNumbers.ChildCount[nil]) + ' new items');
  end;  
end;

procedure TfrmSIMEdit.SendAllContactstoPhonebook1Click(Sender: TObject);
var
  Item: PSIMData;
  Node,NewNode: PVirtualNode;
  Contact: PContactData;
  NewCnt,UpdCnt: cardinal;
  s: WideString;
  j: integer;
begin
  NewCnt := 0;
  UpdCnt := 0;
  try
    Node := ListNumbers.GetFirst;
    while Node <> nil do
    try
      item := ListNumbers.GetNodeData(Node);
      s := Copy(item.pnumb,1,Form1.frmSyncPhonebook.FMaxTellen);
      if Form1.frmSyncPhonebook.FindContact(item.cname,contact) then begin
        { Check if the number already exists for that contact }
        if GetContactPhoneType(contact,s) = '' then
          // TODO: Add wizard for asking where to store new number 
          if MessageDlg('(TODO: add wizard here) Contact '+item.cname+' already exists.'#13#13+
            'Do you want to replace its Phonebook default number ['+
            GetContactDefPhone(contact) + '] with current SIM number [' + s + '] ?',
            mtConfirmation, [mbYes, mbNO], 0) = mrYes then begin
            ReplaceContactDefPhone(contact,s);
            contact^.StateIndex := 1; // contact modified
            inc(UpdCnt);
          end;
      end
      else begin
        NewNode := Form1.frmSyncPhonebook.ListContacts.AddChild(nil);
        contact := Form1.frmSyncPhonebook.ListContacts.GetNodeData(NewNode);
        FillChar(contact^,SizeOf(contact^),0);
        contact^.cell := item.pnumb;
        contact^.StateIndex := 0; // new contact
        s := Copy(item.cname,1,Form1.frmSyncPhonebook.FMaxNameLen+byte(Pos(' ',s) <> 0));
        j := Pos(' ',s);
        if j = 0 then
          contact^.name := s
        else begin
          contact^.name := Copy(s,1,j-1);
          contact^.surname := Copy(s,j+1,Length(s)-j);
        end;
        inc(NewCnt);
      end;
    finally
      Node := ListNumbers.GetNext(Node);
    end;
  finally
    if (NewCnt + UpdCnt) <> 0 then Form1.UpdateMEPhonebook;
    Form1.Status('Copy to Phonebook: ' + IntToStr(NewCnt)+' new, ' + IntToStr(UpdCnt) + ' modified and ' +
      IntToStr(ListNumbers.ChildCount[nil] - NewCnt - UpdCnt) + ' items skipped');
  end;
end;

function TfrmSIMEdit.IsMEMode: boolean;
begin
  Result := Form1.Explorer.Selected = Form1.FNodeContactsME;
end;

procedure TfrmSIMEdit.UpdatePhonebook;
begin
  if IsMEMode then Form1.UpdateMEPhonebook else Form1.UpdateSMPhonebook;
end;

function TfrmSIMEdit.IsRendered: boolean;
const
  LastIsME: boolean = False;
begin
  Result := (LastIsME = IsMEMode) and FRendered;
  LastIsME := IsMEMode;
end;

procedure TfrmSIMEdit.CheckForChanges;
var
  Modified: boolean;
  Contact: PSIMData;
  Node :PVirtualNode;
begin
  Modified := False;
  Node := ListNumbers.GetFirst;
  while Node <> nil do
  try
    Contact := ListNumbers.GetNodeData(Node);
    if Contact^.imageindex <> 3 then begin
      Modified := True;
      break;
    end;
  finally
    Node := ListNumbers.GetNext(Node);
  end;
  if Modified then UpdateChanged1.Click;
end;

procedure TfrmSIMEdit.ListNumbersIncrementalSearch(
  Sender: TBaseVirtualTree; Node: PVirtualNode;
  const SearchText: WideString; var Result: Integer);
var
  SIM: PSIMData;
  Text: WideString;
begin
  SIM := Sender.GetNodeData(Node);
  Text := Copy(SIM.cname,1,Length(SearchText));
  Result := WideCompareText(SearchText,Text);
end;

end.

